#include <stdio.h>
#include <stdlib.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include <string.h>
#include <stdlib.h>

#define SCREEN_WIDTH    400
#define SCREEN_HEIGHT   400

double table[2][SCREEN_HEIGHT][SCREEN_WIDTH];
double nextTable[2][SCREEN_HEIGHT][SCREEN_WIDTH];

double DA = 1.0;
double DB = 0.5;
double f = 0.055;
double k = 0.062;

double Laplacian(int tab, int x, int y)
{
    static const double coef[3][3] =
    {
        {0.05, 0.2,  0.05},
        {0.2,  -1.0, 0.2},
        {0.05, 0.2,  0.05}
    };

    double sum = 0;

    for (int dy = -1; dy <= 1; dy++)
        for (int dx = -1; dx <= 1; dx++)
        {
           int posX = (x + dx + SCREEN_WIDTH) % SCREEN_WIDTH;
           int posY = (y + dy + SCREEN_HEIGHT) % SCREEN_HEIGHT;
           double c = coef[dy + 1][dx + 1];
           sum += c * table[tab][posY][posX];
        }

    return sum;
}

float tim = 0;
int cnt = 0;
void update()
{
    sys.StartPerfCounter();
    for (int y = 0; y < SCREEN_HEIGHT; ++y)
        for (int x = 0; x < SCREEN_WIDTH; ++x)
        {
            double A = table[0][y][x];
            double B = table[1][y][x];

            nextTable[0][y][x] = A + DA * Laplacian(0, x, y) - A * B * B + f * (1.0 - A);
            nextTable[1][y][x] = B + DB * Laplacian(1, x, y) + A * B * B - (k + f) * B;
        }

    memcpy(table, nextTable, SCREEN_WIDTH * SCREEN_HEIGHT * 2 * sizeof(double));

    cnt++;
    if (cnt < 2000)
        tim += sys.StopPerfCounter();
    else if (cnt == 2000)
        printf("%f\n", tim);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Reaction/Diffusion 2", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    // initialize the table
    for (int y = 0; y < SCREEN_HEIGHT; ++y)
        for (int x = 0; x < SCREEN_WIDTH; ++x)
        {
            table[0][y][x] = 1.0;
            table[1][y][x] = 0.0;
        }

    for (int y = 0; y < 10; ++y)
        for (int x = 0; x < 10; ++x)
        {
            int posX = SCREEN_WIDTH  / 2 + x - 5;
            int posY = SCREEN_HEIGHT / 2 + y - 5;
            table[1][posY][posX] = 1.0;
        }

    while (sys.isQuitRequested() == false)
    {
        // draw the current table
        for (int y = 0; y < SCREEN_HEIGHT; ++y)
            for (int x = 0; x < SCREEN_WIDTH; ++x)
            {
                Color col(table[0][y][x] * 255,
                          0,
                          table[1][y][x] * 255);
                gfx.setPixel(x, y, col);
            }

        gfx.render();
        sys.processEvents();

        // update the table
        update();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
